(ns com.aurellem.gb.pokedex
  (:use (com.aurellem.gb gb-driver util species))
  (:import [com.aurellem.gb.gb_driver SaveState]))

(def pokedex-seen-start-address 0xD309)

(def pokedex-captured-start-address 0xD2F6)

(defn pokedex-offset [capture? species-num]
  (+ (if capture?
       pokedex-captured-start-address
       pokedex-seen-start-address)
     (int (/ species-num 8))))
  
(defn read-pokedex
  ([^SaveState state capture? species-num]
     (bit-test
      (aget (memory state)
            (pokedex-offset capture? species-num))
      (rem species-num 8))))

(defn set-pokedex
  ([^SaveState state capture? set? species-num]
     (let [target (pokedex-offset capture? species-num)
           current-bits
           (aget (memory state)
                 (pokedex-offset capture? species-num))
           bit-offset (rem species-num 8)]
       (set-memory
        state target
        (if set?
          (bit-set   current-bits bit-offset)
          (bit-clear current-bits bit-offset))))))

(defn read-pokedex-seen
  ([^SaveState state species-num]
     (read-pokedex state false species-num))
  ([species-num]
     (read-pokedex-seen @current-state species-num)))

(defn set-pokedex-seen
  ([^SaveState state species-num]
     (set-pokedex state false species-num))
  ([species-num]
     (set-pokedex-seen @current-state species-num)))

(defn read-pokedex-captured
  ([^SaveState state species-num]
     (read-pokedex state true species-num))
  ([species-num]
     (read-pokedex-captured @current-state species-num)))

(defn set-pokedex-captured
  ([^SaveState state species-num]
     (set-pokedex state true species-num))
  ([species-num]
     (set-pokedex-captured @current-state species-num)))

(defn print-pokedex
  ([^SaveState state]
     (println "+-----------+-------+-------+")
     (println "|Pokemon    | Seen? | Capt? |")
     (println "+-----------+-------+-------+")
     (dorun
      (map
       #(printf "|%-11s|%s|%s|\n" %1 %2 %3)
       (map #(.toUpperCase (.substring (str %) 1))
            (map pokedex-num->species-name (range 1 152)))
       (map #(if % "   X   " "       ")
            (map read-pokedex-seen (range 152)))
       (map #(if % "   X   " "       ")
            (map read-pokedex-captured (range 152)))))
     (println "+-----------+-------+-------+"))

  ([] (print-pokedex @current-state)))
     